/*--------------  FrvBuf.c by B.Mours Dec  4, 2001---------------------------*/

#include "FrvBuf.h"
#include <string.h>

#define FRV10E9 1000000000

/*------------------------------------------------------------- FrvBufFree --*/
void  FrvTimeAdd(unsigned int *a, unsigned int *b, unsigned int *c)
/*---------------------------------------------------------------------------*/
{
  c[0] = a[0]+b[0];  /*--- seconds field */
  c[1] = a[1]+b[1];  /*--- nanosecond field */

  if(c[1] >= FRV10E9)
    {c[1] -= FRV10E9;
     c[0] ++;}
   
 return;
}
/*------------------------------------------------------------- FrvBufFree --*/
void  FrvTimeAddOffset(unsigned int *tIn, double offset, unsigned int *tOut)
/*---------------------------------------------------------------------------*/
{unsigned int tOff[2];

 tOff[0] = (int) offset;
 tOff[1] = 1.e9*(offset - (double) tOff[0]);
   
 tOut[0] = tIn[0]+tOff[0];  /*--- seconds field */
 tOut[1] = tIn[1]+tOff[1];  /*--- nanosecond field */

 if(tOut[1] >= FRV10E9)
   {tOut[1] -= FRV10E9;
    tOut[0] ++;}
   
 return;
}
/*------------------------------------------------------------- FrvBufFree --*/
void  FrvTimeSubOffset(unsigned int *tIn, double offset, unsigned int *tOut)
/*---------------------------------------------------------------------------*/
{unsigned int tOff[2];

 tOff[0] = (int) offset;
 tOff[1] = 1.e9*(offset - (double) tOff[0]);
   
 if(tOff[1] <= tIn[1] )
   {tOut[0] = tIn[0]-tOff[0];  /*--- seconds field */
    tOut[1] = tIn[1]-tOff[1];} /*--- nanosecond field */
 else
   {tOut[0] = tIn[0]-tOff[0] - 1;  /*--- seconds field */
    tOut[1] = FRV10E9 + tIn[1]-tOff[1];} /*--- nanosecond field */

 return;
}
/*------------------------------------------------------------- FrvBufFree --*/
void  FrvBufFree(struct FrvBuf* buffer)
/*---------------------------------------------------------------------------*/
{
  buffer->input->data = buffer->work;
  FrVectFree(buffer->input);

  buffer->output->data = malloc(1);
  FrVectFree(buffer->output);

  free(buffer);

  return;
}

/*------------------------------------------------------------ FrvBufFeed ---*/
int FrvBufFeed(FrvBuf *buffer, 
               FrVect *vect)
/*---------------------------------------------------------------------------*/
/*  Copy the data in the output structure 
  Returns 0 if at least one output is ready
	  1 if more call to FrvBufFeed are needed
          2 in case of error                
          3 in case of error due to timing missmatch                         */
/*---------------------------------------------------------------------------*/
{unsigned int vTime[2], vectSize, workSize, wSize;
 double dt;

 if(buffer == NULL) return(2);
 if(vect == NULL)   return(2);

          /*----------------------------------- init the object if needed ---*/

 if(buffer->work == NULL)
   {if(FrvBufInit(buffer, vect) != 0) return(2);}

          /*---------------shift existing data to remove the starting gap ---*/

 wSize = buffer->output->wSize;

 if(buffer->first != 0)
   {memmove(buffer->work, buffer->work+buffer->first*wSize,
                       (buffer->last - buffer->first)*wSize);

    buffer->last -= buffer->first;
    buffer->first = 0;}

           /*--------------------------- Increase working space if needed ---*/

 vectSize = vect->nData / buffer->decimate;
 workSize = buffer->last + vectSize;

 if(workSize > buffer->workSize) {
     buffer->work = realloc(buffer->work, workSize*wSize);
     if(buffer->work  == NULL) return(1);
     buffer->workSize = workSize;
     buffer->first = 0;}
  
            /*---- convert the input vector if needed and copy the data------*/
            /*--------------------- at the end of the buffer ----------------*/

 buffer->input->data = buffer->work + buffer->last * wSize;

 if(buffer->decimate != 1)
    {FrVectMap(buffer->input);
     buffer->input->nData = vect->nData;
     if(FrVectDecimate(vect, buffer->decimate, buffer->input)==NULL) return(2);}
 else if(vect->type != buffer->input->type)
    {FrVectMap(buffer->input);
     buffer->input->nData = vect->nData;
     if(FrvCopyTo(vect, 1., buffer->input) == NULL) return(2);}
 else
    {memcpy(buffer->input->data, vect->data, vectSize*wSize);}
 
 buffer->last += vectSize;

         /*--------------------------------------- set timing information ---*/

 FrvTimeAddOffset(&(vect->GTimeS), vect->startX[0], vTime);

        /*------------check the timing accuray to the sampling frequency-----*/

 if((vect->GTimeS > 0) && (buffer->lastGTimeS > 0))
    {dt = buffer->lastGTimeS + 1.e-9*buffer->lastGTimeN -
                   (vTime[0] + 1.e-9*vTime[1]);
     if(dt < 0.) dt = -dt;
     if(dt > vect->dx[0]) return(3);}

 FrvTimeAddOffset(vTime, vect->nData*vect->dx[0], &(buffer->lastGTimeS));

         /*------------------------------------- is the data ready to use?---*/

 if(buffer->last < buffer->readyToUse) return(1);

 return(0);
}


/*------------------------------------------------------------ FrvBufGetNext-*/
int FrvBufGetNext(FrvBuf *buffer)
/*---------------------------------------------------------------------------*/
/*  Prepare the next output buffer 
    Returns: 1 if a call to FrvBufFeed is needed or in case of error  
             0 if a vector is available                                      */
/*---------------------------------------------------------------------------*/
{double offset;

 if(buffer == NULL) return(1);
 if(buffer->work == NULL) return(1);

             /*----------------------- not enough data in the ouput buffer---*/

 if(buffer->last-buffer->first < buffer->readyToUse)  return(1);

             /*-------------------------------- prepare the output buffer----*/

 buffer->output->data = buffer->work + buffer->first*buffer->output->wSize;
 FrVectMap(buffer->output);

             /*------------------------------ set the output vector time ----*/

 offset = (buffer->last - buffer->first)*buffer->output->dx[0];
 FrvTimeSubOffset(&(buffer->lastGTimeS), offset, &(buffer->output->GTimeS));

            /*-------------------- shift the start index for the next use ---*/

 buffer->first += buffer->step;

 return(0);
}

/*------------------------------------------------------------- FrvBufInit --*/
int FrvBufInit(FrvBuf *buffer,
               FrVect *vect)
/*---------------------------------------------------------------------------*/
/* Returns 0 in case success; 1 in case of problem.                          */
/*---------------------------------------------------------------------------*/
{
 if(vect == NULL) return(1);

 if(buffer->nDataOut < 1) buffer->nDataOut = vect->nData;
 if(buffer->outType  < 0) buffer->outType  = vect->type;

       /*---- create the input vector(the space used is the working space)---*/

 buffer->input  = FrVectNew1D(vect->name,  buffer->outType, 
                              vect->nData /buffer->decimate,
                              vect->dx[0], vect->unitX[0], vect->unitY);
 if(buffer->input  == NULL) return(1);

 free(buffer->input->data);
 buffer->input->data = NULL;

       /*---- create the output vector(the space used is the working space)---*/

 buffer->output = FrVectNew1D(vect->name,  buffer->outType, buffer->nDataOut,
                              vect->dx[0], vect->unitX[0], vect->unitY);
 if(buffer->output == NULL) return(1);

 free(buffer->output->data);
 buffer->output->data = NULL;
 buffer->output->startX[0] = 0.;

      /*---------------------------------------allocate the working space ---*/

 if(buffer->step > buffer->nDataOut)
      {buffer->workSize = buffer->step     + buffer->delay + vect->nData;}
 else {buffer->workSize = buffer->nDataOut + buffer->delay + vect->nData;}

 buffer->work = malloc(buffer->workSize * buffer->output->wSize);
 if(buffer->work  == NULL) return(1);

 buffer->readyToUse = buffer->nDataOut + buffer->delay;
 buffer->first = 0;
 buffer->last  = 0;

         /*---------------------------------------------- init timing info ---*/

 FrvTimeAddOffset(&(vect->GTimeS), vect->startX[0], &(buffer->lastGTimeS));

 return(0);
}

/*-------------------------------------------------------------- FrvBufNew --*/
FrvBuf* FrvBufNew(int nDataOut,
                  int step,
                  int outType,
                  int decimate,
                  int delay)
/*---------------------------------------------------------------------------*/
/* Create the buffer structure                                               */
/*---------------------------------------------------------------------------*/
{FrvBuf *buffer;

 buffer = (struct FrvBuf *) (calloc(1,sizeof(struct FrvBuf)));
 if(buffer == NULL) return(NULL);
 
 buffer->nDataOut = nDataOut;
 buffer->step     = step;

 if((outType != -1) && 
    (outType != FR_VECT_2S) && 
    (outType != FR_VECT_4S) && 
    (outType != FR_VECT_4R) && 
    (outType != FR_VECT_8R)) return(NULL);
 buffer->outType  = outType;

 if(decimate < 1)   decimate = 1;
 buffer->decimate = decimate;

 if(delay < 0) delay = 0;
 buffer->delay  = delay;

 return(buffer);
}

/*------------------------------------------------------- FrvBufStillToGo ---*/
int FrvBufStillToGo(FrvBuf *buffer)
/*---------------------------------------------------------------------------*/
/*  This function returns the number of data sample to be given to the buffer
    in order to full up the buffer and to be able to output a vector         */
/*---------------------------------------------------------------------------*/
{int stg;

 if(buffer == NULL) return(-1);

 stg = (buffer->readyToUse - buffer->last) * buffer->decimate;
 if(stg < 0) stg = 0;

 return(stg);
}
