/*  FrvChi2.c by  B.Mours Aug 15, 2000           */


#include "FrvChi2.h"
#include <string.h>
#include <math.h>

/*-------------------------------------------------------------- FrvChi2Ini -*/
int FrvChi2Ini(FrvChi2 *chi2, FrVect *vect)
/*---------------------------------------------------------------------------*/
/*   Returns: 0 in case of probleme, 1 otherwise                             */
/*---------------------------------------------------------------------------*/
{int i, nData;
 double dnu;

 if(chi2 == NULL) return(0);
 if(vect == NULL) return(0);
       
                /*------- init buffer object -----*/

 if(FrvBufInit(chi2->buffer, vect) == 0) return(0);

                /*------- init FFT object -----*/

 if(FrvRFFTInit(chi2->fft, chi2->buffer->output) == 0) return(0);
 chi2->fft->decay = chi2->decay;

                /*--- save vector name -----*/

 if(FrStrCpy(&(chi2->name),vect->name) == NULL) return(0);

               /*--------- create output structures -----*/

 nData = chi2->buffer->output->nData;
 dnu = chi2->fft->amplitude->dx[0];
 chi2->amplitude2 = FrVectNew1D(vect->name,FR_VECT_8R, nData/2, dnu,
		                "frequency[Hz]","power**2");
 if(chi2->amplitude2 == NULL)   return(0);
    
 for (i=0; i<chi2->amplitude2->nData; i++)
      {chi2->amplitude2->dataD[i] = 0.;}

             /*---- create debug structures ---------------*/

 if(chi2->threshold > 0.)
   {chi2->amplitude  = FrVectNew1D(vect->name,FR_VECT_8R, nData/2, dnu,
                                "frequency[Hz]","amplitude");
    chi2->amplitudeA = FrVectNew1D(vect->name,FR_VECT_8R, nData/2, dnu,
                                "frequency[Hz]","amplitudeA");
    chi2->rms2       = FrVectNew1D(vect->name,FR_VECT_8R, nData/2, dnu,
                                "frequency[Hz]","rms**2");
    chi2->residual   = FrVectNew1D(vect->name,FR_VECT_8R, nData/2, dnu,
                                "frequency[Hz]","residual");
    chi2->dataIn = FrVectCopy(chi2->buffer->output);

    if((chi2->amplitude  == NULL) ||
       (chi2->amplitudeA == NULL) ||
       (chi2->rms2       == NULL) ||
       (chi2->residual   == NULL) ||
       (chi2->dataIn     == NULL)) return(0);}

 /*----- initialize the power boundary for power spectrum ---*/

 chi2->ip0 = chi2->freq0/dnu;
 chi2->ip1 = chi2->freq1/dnu;
 chi2->ip2 = chi2->freq2/dnu;
 chi2->ip3 = nData/2;
 if(chi2->ip0 > nData/2) chi2->ip0 = nData/2;
 if(chi2->ip1 > nData/2) chi2->ip1 = nData/2;
 if(chi2->ip2 > nData/2) chi2->ip2 = nData/2;

 return(1);
}
/*-------------------------------------------------------------- FrvChi2New -*/
FrvChi2* FrvChi2New(int outSize, int window, double threshold)
/*---------------------------------------------------------------------------*/
{FrvChi2 *chi2;
 int step;

 chi2 = (FrvChi2 *) (calloc(1, sizeof(FrvChi2)));
 if(chi2 == NULL) return(NULL);

 chi2->outSize   = outSize;
 chi2->window    = window;
 chi2->threshold = threshold;
 chi2->decay   = .99;
 chi2->nCall   = 0;
 chi2->freq0 = 32.;
 chi2->freq1 = 128.;
 chi2->freq2 = 1024.;
                        
            /*------- create FFT buffer object -----*/

 if(chi2->window == 0)
     {step = chi2->outSize;}
 else{step = .9*chi2->outSize;} /* Since we are short in CPU, do a partiasl shift*/

 chi2->buffer = FrvBufNew(chi2->outSize, step, 0, 0, 0);

 if(chi2->buffer == NULL) return(NULL);

            /*------- create FFT object -----*/

 if(chi2->window == 0)
     {chi2->fft = FrvRFFTNew("AP",0,0);}
 else{chi2->fft = FrvRFFTNew("APH",0,0);}

 if(chi2->fft == NULL) return(NULL);

 return(chi2);
}
/*-------------------------------------------------------------- FrvChi2Proc-*/
FrvChi2* FrvChi2Proc(FrvChi2 *chi2, FrVect *vect)
/*---------------------------------------------------------------------------*/
/*   Applies the CHI2 algorithm "chi2" to the vector "vect"                  */
/*   Returns the result in the FrvChi2 structure as output                   */
/*   If on input chi2=0, builds a chi2 structure from the vector vect        */
/*---------------------------------------------------------------------------*/
{
 int isegment, i;
 double *a, *aA, *a2, chi2Val, chi2Max, chi2Freq, 
         decay, res, res2, rms2;

 if(chi2 == NULL)  return(NULL);
 if(vect == NULL)  return(NULL);

          /*------------ initialized chi2 object if needed ------------------*/

 if(chi2->amplitude2 == NULL)
   {if(FrvChi2Ini(chi2,vect) == 0) return(0);}

          /*---------------------- feed data --------------------------------*/

 if(FrvBufFeed(chi2->buffer,vect) == 0) return(0);     
 
          /*-------------------------set decay to 0 for the first call ------*/

 if(chi2->nCall == 0)
     {decay = 0.;}
 else{decay = 1.-1./chi2->nCall;}
 if(chi2->decay < decay) decay = chi2->decay;

 chi2->nCall++;

          /*---------------------------- now loop over the sub samples ------*/

 chi2Max  = 0;
 isegment = 0;

 chi2->power1 = 0.;
 chi2->power2 = 0.;
 chi2->power3 = 0.;

 while(FrvBufGetNext(chi2->buffer) == 0)

   {isegment++;
                
               /*------------------------------- compute FFT ----------------*/

    FrvRFFTFor(chi2->fft, chi2->buffer->output);

              /*-------------------------------- compute chisquare ----------*/

    a  = chi2->fft->amplitude->dataD;
    aA = chi2->fft->amplitudeA->dataD;
    a2 = chi2->amplitude2->dataD;

    chi2Val  = 0.;
    chi2Freq = 0.;
    for(i=0; i<chi2->amplitude2->nData; i++)  
      {a2[i] = chi2->decay *a2[i] + (1.-chi2->decay)*a[i]*a[i];
       rms2 = a2[i] - aA[i]*aA[i];
       if(rms2 > 0.)
            {res  = a[i]-aA[i];
             res2 = res*res/rms2;}
       else {res2 = 0.;}
       chi2Val  +=   res2;
       chi2Freq += i*res2;}
 
    if(chi2Val > chi2Max)
      {chi2Max = chi2Val;
       chi2->chi2Time = isegment;   
       chi2->chi2Freq = chi2Freq/chi2Val;

              /*----------- save the loudest segment for debug------*/

       if((chi2Val > chi2->threshold) && (chi2->threshold > 0.))  
          {for(i=0; i<chi2->amplitude2->nData; i++)  
            {chi2->amplitude->dataD[i]  = a[i];
             chi2->amplitudeA->dataD[i] = aA[i];
             chi2->rms2->dataD[i] = a2[i]-aA[i]*aA[i];
             rms2 = a2[i] - aA[i]*aA[i];
             if(rms2 > 0.)
                  {chi2->residual->dataD[i] = (a[i]-aA[i])/ sqrt(rms2);}
             else {chi2->residual->dataD[i] = 0.;}}

           memcpy(chi2->dataIn->data, 
                  chi2->buffer->output->data, chi2->dataIn->nBytes);}}

              /*---------------- compute power in frequency bands ----------*/

    for(i=chi2->ip0; i<chi2->ip1; i++) {chi2->power1 += a[i]*a[i]*i*i;}
    for(i=chi2->ip1; i<chi2->ip2; i++) {chi2->power2 += a[i]*a[i]*i*i;}
    for(i=chi2->ip2; i<chi2->ip3; i++) {chi2->power3 += a[i]*a[i]*i*i;}
  }

 
 chi2->chi2 = chi2Max*2/chi2->amplitude2->nData;   

                     /*------- Set start time for all the series defined --- */

 chi2->amplitude2->GTimeS = vect->GTimeS;
 chi2->amplitude2->GTimeN = vect->GTimeN;

 return(chi2);

}


