#define _POSIX_SOURCE 
/*---------------------------------------------------------------------------*/
/*  FrvCopy.c by B.Mours LAPP (Annecy)/Caltech Nov 16, 2001                  */
/*---------------------------------------------------------------------------*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <ctype.h>
#include "FrvCopy.h"

/*-----------------------------------------------------------------FrvClone--*/
FrVect *FrvClone(FrVect *vectin,
                 char *newName)
/*---------------------------------------------------------------------------*/
/* This function create a new vector (FrVect structure and data area);       */
/* It copy the header information but not the data.                          */
/* If newName = NULL, the original vector name is used.                      */
/* This function returns null in case of problems like malloc failed.        */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 long i;

  if(vectin == NULL) return(NULL);

   vect = (FrVect *) calloc(1,sizeof(FrVect));
   if(vect == NULL)  
        {return(NULL);}
   else {vect->classe = FrVectDef();}

   if(newName != NULL)
        {if(FrStrCpy(&vect->name,      newName) == NULL) return(NULL);}
   else {if(FrStrCpy(&vect->name, vectin->name) == NULL) return(NULL);}

   vect->compress = 0;
   vect->type     = vectin->type;
   vect->nData    = vectin->nData;
   vect->nBytes   = vectin->nBytes;
   vect->nDim     = vectin->nDim;
   vect->nx     = malloc(vectin->nDim*sizeof(long));
   vect->unitX  = malloc(vectin->nDim*sizeof(char *));
   vect->startX = malloc(vectin->nDim*sizeof(double));
   vect->dx     = malloc(vectin->nDim*sizeof(double));
   if(vect->nx     == NULL || 
      vect->unitX  == NULL || 
      vect->startX == NULL || 
      vect->dx     == NULL) return(NULL);

   for(i=0; i<vectin->nDim; i++)
     {vect->nx[i]     = vectin->nx[i];
      vect->startX[i] = vectin->startX[i];
      vect->dx[i]     = vectin->dx[i];
      FrStrCpy(&vect->unitX[i], vectin->unitX[i]);
      if(vect->unitX[i] == NULL && vectin->unitX[i] != NULL) return(NULL);}


   FrStrCpy(&vect->unitY, vectin->unitY);

   vect->wSize = vectin->wSize;
   vect->size = vectin->size;
   vect->data = malloc(vectin->size*vectin->wSize);
   if(vect->data == NULL) 
      {FrError(3,"FrVectCopy","malloc failed");
       return(NULL);}

   FrVectMap(vect);

   vect->GTimeS = vectin->GTimeS;
   vect->GTimeN = vectin->GTimeN;
   vect->next = NULL;

   return(vect);
}

/*----------------------------------------------------------------FrvCopyTo--*/
FrVect *FrvCopyTo(FrVect *vIn,
                  double scale,
                  FrVect *copy)
/*---------------------------------------------------------------------------*/
/* This function copy the data from the vector vIn to the vector copy using  */
/* the scale factor 'scale' and casted to the vector copy type.              */
/* It return NULL in case of error (malloc failed, no input vectors).        */
/*---------------------------------------------------------------------------*/
{int i;

 if(vIn  == NULL)              return(NULL);
 if(copy == NULL)              return(NULL);
 if(copy->nData != vIn->nData) return(NULL);
 
 for(i=0; i<vIn->nDim; i++) {copy->startX[i] = vIn->startX[i];}
 copy->GTimeS = vIn->GTimeS;
 copy->GTimeN = vIn->GTimeN;

                   /*----------------------- copy to D ----------------------*/

 if(copy->type == FR_VECT_8R)  
  {if(scale == 1.)
    {if(vIn->type == FR_VECT_8R)
      {memcpy(copy->dataD, vIn->dataD, vIn->nData*sizeof(double));}
     else if(vIn->type == FR_VECT_C)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->data[i];}}
     else if(vIn->type == FR_VECT_2S)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataS[i];}}
     else if(vIn->type == FR_VECT_8R)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_4R)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_4S)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataI[i];}}
     else if(vIn->type == FR_VECT_8S)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataL[i];}}
     else if(vIn->type == FR_VECT_C8)
      {for(i=0; i<2*copy->nData; i++){copy->dataD[i] = vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_2U)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataUS[i];}}
     else if(vIn->type == FR_VECT_4U)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataUI[i];}}
     else if(vIn->type == FR_VECT_8U)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataUL[i];}}
     else if(vIn->type == FR_VECT_1U)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = vIn->dataU[i];}}}
   else
    {if(vIn->type == FR_VECT_8R) 
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_C)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->data[i];}}
     else if(vIn->type == FR_VECT_2S)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataS[i];}}
     else if(vIn->type == FR_VECT_8R)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_4R)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_4S)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataI[i];}}
     else if(vIn->type == FR_VECT_8S)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataL[i];}}
     else if(vIn->type == FR_VECT_C8)
      {for(i=0; i<2*copy->nData; i++){copy->dataD[i] = scale*vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_2U)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataUS[i];}}
     else if(vIn->type == FR_VECT_4U)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataUI[i];}}
     else if(vIn->type == FR_VECT_8U)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataUL[i];}}
     else if(vIn->type == FR_VECT_1U)
      {for(i=0; i<copy->nData; i++)  {copy->dataD[i] = scale*vIn->dataU[i];}}}}

                   /*----------------------- copy to F ----------------------*/

 else if(copy->type == FR_VECT_4R)  
  {if(scale == 1.)
    {if(vIn->type == FR_VECT_4R)
     {memcpy(copy->dataF, vIn->dataF, copy->nData * sizeof(float));}
     else if(vIn->type == FR_VECT_C)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->data[i];}}
     else if(vIn->type == FR_VECT_2S)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->dataS[i];}}
     else if(vIn->type == FR_VECT_8R)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_4S)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->dataI[i];}}
     else if(vIn->type == FR_VECT_8S)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->dataL[i];}}
     else if(vIn->type == FR_VECT_C8)
      {for(i=0; i<2*copy->nData; i++){copy->dataF[i] = vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_C16)
      {for(i=0; i<2*copy->nData; i++){copy->dataF[i] = vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_2U)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->dataUS[i];}}
     else if(vIn->type == FR_VECT_4U)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->dataUI[i];}}
     else if(vIn->type == FR_VECT_8U)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->dataUL[i];}}
     else if(vIn->type == FR_VECT_1U)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = vIn->dataU[i];}}}
   else
     {if(vIn->type == FR_VECT_4R)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_C)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->data[i];}}
     else if(vIn->type == FR_VECT_2S)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataS[i];}}
     else if(vIn->type == FR_VECT_8R)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_4S)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataI[i];}}
     else if(vIn->type == FR_VECT_8S)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataL[i];}}
     else if(vIn->type == FR_VECT_C8)
      {for(i=0; i<2*copy->nData; i++){copy->dataF[i] = scale*vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_C16)
      {for(i=0; i<2*copy->nData; i++){copy->dataF[i] = scale*vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_2U)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataUS[i];}}
     else if(vIn->type == FR_VECT_4U)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataUI[i];}}
     else if(vIn->type == FR_VECT_8U)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataUL[i];}}
     else if(vIn->type == FR_VECT_1U)
      {for(i=0; i<copy->nData; i++)  {copy->dataF[i] = scale*vIn->dataU[i];}}}}

                   /*----------------------- copy to I ----------------------*/

 else if(copy->type == FR_VECT_4S)  
  {if(scale == 1.)
    {if(vIn->type == FR_VECT_4S)
      {memcpy(copy->data, vIn->data, copy->nData *sizeof(int));}
     else if(vIn->type == FR_VECT_C)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->data[i];}}
     else if(vIn->type == FR_VECT_2S)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->dataS[i];}}
     else if(vIn->type == FR_VECT_8R)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_4R)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_8S)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->dataL[i];}}
     else if(vIn->type == FR_VECT_C8)
      {for(i=0; i<2*copy->nData; i++){copy->dataI[i] = vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_C16)
      {for(i=0; i<2*copy->nData; i++){copy->dataI[i] = vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_2U)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->dataUS[i];}}
     else if(vIn->type == FR_VECT_4U)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->dataUI[i];}}
     else if(vIn->type == FR_VECT_8U)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->dataUL[i];}}
     else if(vIn->type == FR_VECT_1U)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = vIn->dataU[i];}}}
   else
    {if(vIn->type == FR_VECT_4S)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataI[i];}}
     else if(vIn->type == FR_VECT_C)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->data[i];}}
     else if(vIn->type == FR_VECT_2S)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataS[i];}}
     else if(vIn->type == FR_VECT_8R)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_4R)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_8S)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataL[i];}}
     else if(vIn->type == FR_VECT_C8)
      {for(i=0; i<2*copy->nData; i++){copy->dataI[i] = scale*vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_C16)
      {for(i=0; i<2*copy->nData; i++){copy->dataI[i] = scale*vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_2U)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataUS[i];}}
     else if(vIn->type == FR_VECT_4U)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataUI[i];}}
     else if(vIn->type == FR_VECT_8U)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataUL[i];}}
     else if(vIn->type == FR_VECT_1U)
      {for(i=0; i<copy->nData; i++)  {copy->dataI[i] = scale*vIn->dataU[i];}}}}

                   /*----------------------- copy to S ----------------------*/

 else if(copy->type == FR_VECT_2S)  
  {if(scale == 1.)
    {if(vIn->type == FR_VECT_2S)
      {memcpy(copy->dataS, vIn->dataS, copy->nData*sizeof(short));}
     else if(vIn->type == FR_VECT_C)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->data[i];}}
     else if(vIn->type == FR_VECT_8R)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_4R)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_4S)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->dataI[i];}}
     else if(vIn->type == FR_VECT_8S)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->dataL[i];}}
     else if(vIn->type == FR_VECT_C8)
      {for(i=0; i<2*copy->nData; i++){copy->dataS[i] = vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_C16)
      {for(i=0; i<2*copy->nData; i++){copy->dataS[i] = vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_2U)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->dataUS[i];}}
     else if(vIn->type == FR_VECT_4U)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->dataUI[i];}}
     else if(vIn->type == FR_VECT_8U)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->dataUL[i];}}
     else if(vIn->type == FR_VECT_1U)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = vIn->dataU[i];}}}
   else
    {if(vIn->type == FR_VECT_2S)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataS[i];}}
     else if(vIn->type == FR_VECT_C)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->data[i];}}
     else if(vIn->type == FR_VECT_8R)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_4R)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_4S)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataI[i];}}
     else if(vIn->type == FR_VECT_8S)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataL[i];}}
     else if(vIn->type == FR_VECT_C8)
      {for(i=0; i<2*copy->nData; i++){copy->dataS[i] = scale*vIn->dataF[i];}}
     else if(vIn->type == FR_VECT_C16)
      {for(i=0; i<2*copy->nData; i++){copy->dataS[i] = scale*vIn->dataD[i];}}
     else if(vIn->type == FR_VECT_2U)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataUS[i];}}
     else if(vIn->type == FR_VECT_4U)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataUI[i];}}
     else if(vIn->type == FR_VECT_8U)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataUL[i];}}
     else if(vIn->type == FR_VECT_1U)
      {for(i=0; i<copy->nData; i++)  {copy->dataS[i] = scale*vIn->dataU[i];}}}}

                     /*------------------ unsuported type -------------------*/

 else
   {return(NULL);}

 return(copy);
}
/*---------------------------------------------------------------FrvCopyToD--*/
FrVect *FrvCopyToD(FrVect *vIn,
                   double scale,
                   char *newName)
/*---------------------------------------------------------------------------*/
/* This function creates a new vector of type double. The data are copy using*/
/* the scale factor 'scale' and casted to the proper type. The new vector    */
/* has the same name as the original one except if a newName is provided     */
/*---------------------------------------------------------------------------*/
{FrVect *vect;

 if(vIn == NULL) return(NULL);

 if(newName == NULL) newName = vIn->name;

 vect = FrVectNew1D(newName, FR_VECT_8R, vIn->nData, vIn->dx[0], 
                    vIn->unitX[0], vIn->unitY);
 vect = FrvCopyTo(vIn, scale, vect);

 return(vect);
}
/*---------------------------------------------------------------FrvCopyToF--*/
FrVect *FrvCopyToF(FrVect *vIn,
                   double scale,
                   char *newName)
/*---------------------------------------------------------------------------*/
/* This function creates a new vector of type float. The data are copy using */
/* the scale factor 'scale' and casted to the proper type. The new vector    */
/* has the same name as the original one except if a newName is provided     */
/*---------------------------------------------------------------------------*/
{FrVect *vect;

 if(vIn == NULL) return(NULL);

 if(newName == NULL) newName = vIn->name;

 vect = FrVectNew1D(newName, FR_VECT_4R, vIn->nData, vIn->dx[0], 
                    vIn->unitX[0], vIn->unitY);
 vect = FrvCopyTo(vIn, scale, vect);

 return(vect);
}
/*---------------------------------------------------------------FrvCopyToI--*/
FrVect *FrvCopyToI(FrVect *vIn,
                   double scale,
                   char *newName)
/*---------------------------------------------------------------------------*/
/* This function creates a new vector of type int. The data are copy using   */
/* the scale factor 'scale' and casted to the proper type. The new vector    */
/* has the same name as the original one except if a newName is provided     */
/*---------------------------------------------------------------------------*/
{FrVect *vect;

 if(vIn == NULL) return(NULL);

 if(newName == NULL) newName = vIn->name;

 vect = FrVectNew1D(newName, FR_VECT_4S, vIn->nData, vIn->dx[0], 
                    vIn->unitX[0], vIn->unitY);
 vect = FrvCopyTo(vIn, scale, vect);

 return(vect);
}
/*---------------------------------------------------------------FrvCopyToS--*/
FrVect *FrvCopyToS(FrVect *vIn,
                   double scale,
                   char *newName)
/*---------------------------------------------------------------------------*/
/* This function creates a new vector of type short. The data are copy using */
/* the scale factor 'scale' and casted to the proper type. The new vector    */
/* has the same name as the original one except if a newName is provided     */
/*---------------------------------------------------------------------------*/
{FrVect *vect;

 if(vIn == NULL) return(NULL);

 if(newName == NULL) newName = vIn->name;

 vect = FrVectNew1D(newName, FR_VECT_2S, vIn->nData, vIn->dx[0], 
                    vIn->unitX[0], vIn->unitY);
 vect = FrvCopyTo(vIn, scale, vect);

 return(vect);
}

/*-------------------------------------------------------------FrvDecimateD--*/
FrVect *FrvDecimateD(FrVect *vIn,
                     int nGroup,
                     char *newName)
/*---------------------------------------------------------------------------*/
/* This function decimates the data from the vector vIn by averaging nGroup  */
/* values together. The result is put in a new vector named 'newName' of type*/
/* double. The size of the output vector is nGroup time smaller than the size*/
/* of the input vector vect. If newName = NULL, the name of the output       */
/* vector is the same as the input one.                                      */
/*---------------------------------------------------------------------------*/
{FrVect *vect;

 if(vIn == NULL) return(NULL);

 if(newName == NULL) newName = vIn->name;

 vect = FrVectNew1D(newName, FR_VECT_8R, vIn->nData/nGroup, vIn->dx[0]*nGroup, 
                    vIn->unitX[0], vIn->unitY);
 vect = FrVectDecimate(vIn, nGroup, vect);

 return(vect);
}
/*-------------------------------------------------------------FrvDecimateF--*/
FrVect *FrvDecimateF(FrVect *vIn,
                     int nGroup,
                     char *newName)
/*---------------------------------------------------------------------------*/
/* This function decimates the data from the vector vIn by averaging nGroup  */
/* values together. The result is put in a new vector named 'newName' of type*/
/* float. The size of the output vector is nGroup time smaller than the size */
/* of the input vector vect. If newName = NULL, the name of the output       */
/* vector is the same as the input one.                                      */
/*---------------------------------------------------------------------------*/
{FrVect *vect;

 if(vIn == NULL) return(NULL);

 if(newName == NULL) newName = vIn->name;

 vect = FrVectNew1D(newName, FR_VECT_4R, vIn->nData/nGroup, vIn->dx[0]*nGroup, 
                    vIn->unitX[0], vIn->unitY);
 vect = FrVectDecimate(vIn, nGroup, vect);

 return(vect);
}
/*-------------------------------------------------------------FrvDecimateI--*/
FrVect *FrvDecimateI(FrVect *vIn,
                     int nGroup,
                     char *newName)
/*---------------------------------------------------------------------------*/
/* This function decimates the data from the vector vIn by averaging nGroup  */
/* values together. The result is put in a new vector named 'newName' of type*/
/* int. The size of the output vector is nGroup time smaller than the size   */
/* of the input vector vect. If newName = NULL, the name of the output       */
/* vector is the same as the input one.                                      */
/*---------------------------------------------------------------------------*/
{FrVect *vect;

 if(vIn == NULL) return(NULL);

 if(newName == NULL) newName = vIn->name;

 vect = FrVectNew1D(newName, FR_VECT_4S, vIn->nData/nGroup, vIn->dx[0]*nGroup, 
                    vIn->unitX[0], vIn->unitY);
 vect = FrVectDecimate(vIn, nGroup, vect);

 return(vect);
}
/*-------------------------------------------------------------FrvDecimateS--*/
FrVect *FrvDecimateS(FrVect *vIn,
                     int nGroup,
                     char *newName)
/*---------------------------------------------------------------------------*/
/* This function decimates the data from the vector vIn by averaging nGroup  */
/* values together. The result is put in a new vector named 'newName' of type*/
/* short. The size of the output vector is nGroup time smaller than the size */
/* of the input vector vect. If newName = NULL, the name of the output       */
/* vector is the same as the input one.                                      */
/*---------------------------------------------------------------------------*/
{FrVect *vect;

 if(vIn == NULL) return(NULL);

 if(newName == NULL) newName = vIn->name;

 vect = FrVectNew1D(newName, FR_VECT_2S, vIn->nData/nGroup, vIn->dx[0]*nGroup, 
                    vIn->unitX[0], vIn->unitY);
 vect = FrVectDecimate(vIn, nGroup, vect);

 return(vect);
}

