/*------- FrvFilter.h   Nov 1, 2000 (from Siesta file USignal.*)-------------*/

#include "FrvFilter.h"
#include <string.h>
#include <math.h>

/*--------------------------------------------------------- FrvFilter2Free --*/
void  FrvFilter2Free(FrvFilter2* filter)
/*---------------------------------------------------------------------------*/
{
  FrVectFree(filter->output);

  free(filter);

  return;
}
/*---------------------------------------------------------- FrvFilter2Proc--*/
int FrvFilter2Proc(FrvFilter2 *filter, FrVect *vect)
/*---------------------------------------------------------------------------*/
/*   Applies the direct FILTER algorithm "filter" to the vector "vect"       */
/*---------------------------------------------------------------------------*/
{double input;
 int i;

 if(vect    == NULL) return(1);
 if(filter  == NULL) return(2);  
 
 if(filter->output == NULL)
   {if(FrvFilter2Init(filter, vect) == 1) return(3);}

          /*------------------ check that we work on the same buffer length--*/

 if(filter->output->nData != vect->nData) 
   {sprintf(filter->error," nData missmatch:%d %d\n",
                   filter->output->nData, vect->nData);
    return(4);}

         /*-------------------- set start time for all the series defined ---*/

 filter->output->GTimeS  = vect->GTimeS;
 filter->output->GTimeN  = vect->GTimeN;

 input = 0.;
 for(i=0; i<vect->nData; i++)
   {if     (vect->type == FR_VECT_4R) input = vect->dataF[i];
    else if(vect->type == FR_VECT_8R) input = vect->dataD[i];
    else if(vect->type == FR_VECT_2S) input = vect->dataS[i];
    else if(vect->type == FR_VECT_4S) input = vect->dataI[i];
    else if(vect->type == FR_VECT_8S) input = vect->dataL[i];
    else if(vect->type == FR_VECT_C)  input = vect->data[i];
    else if(vect->type == FR_VECT_2U) input = vect->dataUS[i];
    else if(vect->type == FR_VECT_4U) input = vect->dataUI[i];
    else if(vect->type == FR_VECT_8U) input = vect->dataUL[i];
    else if(vect->type == FR_VECT_1U) input = vect->dataU[i];
    filter->reg2 = filter->reg1;
    filter->reg1 = filter->reg0;
    filter->reg0 = input 
                 + filter->beta1 * filter->reg1
                 + filter->beta2 * filter->reg2; 
    filter->output->dataD[i] = filter->alfa0 * filter->reg0
                             + filter->alfa1 * filter->reg1
                             + filter->alfa2 * filter->reg2;}

 filter->nCall++;

 return(0);

}

/*--------------------------------------------------------- FrvFilter2Init --*/
int FrvFilter2Init(FrvFilter2* filter, FrVect *vect)
/*---------------------------------------------------------------------------*/
/*   Initilized the filter object: creat all output structures               */
/*   Returns: 0 in case of success, 1 otherwise                              */
/*---------------------------------------------------------------------------*/
{double fs;

 if(vect == NULL) 
    {sprintf(filter->error,"no input vector");
     return(1);}

 if((vect->type == FR_VECT_C8)  ||
    (vect->type == FR_VECT_C16) ||
    (vect->type == FR_VECT_STRING)) 
    {sprintf(filter->error,"Input type not supported:%d",vect->type);
     return(0);}

 if (filter == NULL) return(1);
 
 if(vect->dx[0] <= 0)
    {sprintf(filter->error,"bad time step:%f",vect->dx[0]);
     return(1);}

 fs = 1. / vect->dx[0];

 filter->alfa2 =   4.*filter->a2*fs*fs  -  2.*filter->a1*fs  +  filter->a0;  
 filter->alfa1 = - 8.*filter->a2*fs*fs  +  2.*filter->a0;  
 filter->alfa0 =   4.*filter->a2*fs*fs  +  2.*filter->a1*fs  +  filter->a0;  
 filter->beta2 =   4.*filter->b2*fs*fs  -  2.*filter->b1*fs  +  filter->b0;  
 filter->beta1 = - 8.*filter->b2*fs*fs  +  2.*filter->b0;  
 filter->beta0 =   4.*filter->b2*fs*fs  +  2.*filter->b1*fs  +  filter->b0;  

 filter->alfa2 =   filter->alfa2 / filter->beta0;
                          
 filter->alfa1 =   filter->alfa1 / filter->beta0;
 filter->alfa0 =   filter->alfa0 / filter->beta0;
 filter->beta2 = - filter->beta2 / filter->beta0;
 filter->beta1 = - filter->beta1 / filter->beta0;

 filter->reg2  = 0.;
 filter->reg1  = 0.;
 filter->reg0  = 0.;

          /*---- init for output space ----*/

 filter->output  = FrVectNew1D(vect->name, FR_VECT_8R, vect->nData, 
                               vect->dx[0], "time [s]",vect->unitY);
 if(filter->output  == NULL) 
    {sprintf(filter->error,"malloc in/out failed:ndata=%d",vect->nData);
     return(1);}

 return(0);

}
/*---------------------------------------------------------- FrvFilter2New --*/
FrvFilter2* FrvFilter2New(double a2,
                          double a1,
                          double a0,
                          double b2,
                          double b1,
                          double b0)
/*---------------------------------------------------------------------------*/
{
 FrvFilter2 *filter;

      /*-------- create the filter structure ----*/

 filter = (FrvFilter2 *) (calloc(1,sizeof(FrvFilter2)));
 if(filter == NULL) return(NULL);
 
 filter->a2 = a2;
 filter->a1 = a1;
 filter->a0 = a0;
 filter->b2 = b2;
 filter->b1 = b1;
 filter->b0 = b0;

 filter->nCall = 0;

 return(filter);
}


