#define _POSIX_SOURCE 
/*---------------------------------------------------------------------------*/
/*  FrvMath.c by I. Ferrante, F. Marion, B.Mours           Dec  3, 2001      */
/*---------------------------------------------------------------------------*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <ctype.h>
#include "FrvMath.h"

/*-------------------------------------------------------------------FrvAdd--*/
FrVect *FrvAdd(FrVect *vect1,
               FrVect *vect2,
               FrVect *vect3,
               char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes vectOut= vect1+vect2. The vector vectOut could be  */
/* vect1 or vect2. If vectOut = NULL, a new one is created named newName.    */
/* It returns, the output vector or NULL in case of error.                   */
/* If vectOut = NULL, the newName argument is ignored.                       */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;

 if(vect1 == NULL) return(NULL);
 if(vect2 == NULL) return(NULL);
 if(vect1->type  != vect2->type)  return(NULL);
 if(vect1->nData != vect2->nData) return(NULL);

 if(vect3 != NULL)
      {if(vect3->type  != vect1->type)  return(NULL);
       if(vect3->nData != vect1->nData) return(NULL);
       vect = vect3;}
 else {vect = FrvClone(vect1,newName);}

 if(vect->type == FR_VECT_C)
    {for(i=0; i<vect->nData; i++)
        {vect->data[i] = vect1->data[i] + vect2->data[i];}}
 else if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataS[i] = vect1->dataS[i] + vect2->dataS[i];}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] + vect2->dataD[i];}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] + vect2->dataF[i];}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataI[i] = vect1->dataI[i] + vect2->dataI[i];}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataL[i] = vect1->dataL[i] + vect2->dataL[i];}}
 else if(vect->type == FR_VECT_C8)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] + vect2->dataF[i];}}
 else if(vect->type == FR_VECT_C16)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] + vect2->dataD[i];}}
 else if(vect->type == FR_VECT_2U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUS[i] = vect1->dataUS[i] + vect2->dataUS[i];}}
 else if(vect->type == FR_VECT_4U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUI[i] = vect1->dataUI[i] + vect2->dataUI[i];}}
 else if(vect->type == FR_VECT_8U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUL[i] = vect1->dataUL[i] + vect2->dataUL[i];}}
 else if(vect->type == FR_VECT_1U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataU[i] = vect1->dataU[i] + vect2->dataU[i];}}

 return(vect);
}

/*------------------------------------------------------------------FrvBias--*/
FrVect *FrvBias(FrVect *vect1, double bias, FrVect *vectOut)
/*---------------------------------------------------------------------------*/
/* This function adds bias to vector vect1 .                                 */
/* If vectOut==NULL, vect1 is modified.  Otherwise, a new vector with name   */
/* newName is created.                                                       */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;

 if(vect1 == NULL) return(NULL);


 if(vectOut != NULL)
      {if(vectOut->type  != vect1->type)  return(NULL);
       if(vectOut->nData != vect1->nData) return(NULL);
       vect = vectOut;}
 else vect=vect1;
 /* output data type not supported! */
 if ((vect->type == FR_VECT_C) ||
     (vect->type == FR_VECT_8U)||
     (vect->type == FR_VECT_4U)||
     (vect->type == FR_VECT_2U)||
     (vect->type == FR_VECT_1U)) return (NULL);


 if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataS[i] = vect1->dataS[i] + bias;}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i]+ bias;}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] + bias;}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataI[i] = vect1->dataI[i]+ bias;}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataL[i] = vect1->dataL[i] + bias;}}
 else if(vect->type == FR_VECT_C8)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] + bias;}}
 else if(vect->type == FR_VECT_C16)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] + bias;}}
 

 return(vect);
}

/*-------------------------------------------------------------FrvCombine2--*/
FrVect *FrvCombine2(double s1,
                    FrVect *vect1,
                    double s2,
                    FrVect *vect2,
                    FrVect *vect3,
                    char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes vect3 = s1*vect1+ s2*vect2 (elements by elements). */
/* The vector vect3 could be vect1 or vect2. If vect3 = NULL, a new one      */
/* is created. Its name is then newName. It returns the vector result or NULL*/
/* in case of error. If vect3 != NULL, the newName argument is ignored.      */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;

 if(vect1 == NULL) return(NULL);
 if(vect2 == NULL) return(NULL);
 if(vect1->type  != vect2->type)  return(NULL);
 if(vect1->nData != vect2->nData) return(NULL);

 if(vect3 != NULL)
      {if(vect3->type  != vect1->type)  return(NULL);
       if(vect3->nData != vect1->nData) return(NULL);
       vect = vect3;}
 else {vect = FrvClone(vect1,newName);}

 if(vect->type == FR_VECT_C)
    {for(i=0; i<vect->nData; i++)
        {vect->data[i] = s1*vect1->data[i] + s2*vect2->data[i];}}
 else if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataS[i] = s1*vect1->dataS[i] + s2*vect2->dataS[i];}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = s1*vect1->dataD[i] + s2*vect2->dataD[i];}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = s1*vect1->dataF[i] + s2*vect2->dataF[i];}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataI[i] = s1*vect1->dataI[i] + s2*vect2->dataI[i];}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataL[i] = s1*vect1->dataL[i] + s2*vect2->dataL[i];}}
 else if(vect->type == FR_VECT_C8)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataF[i] = s1*vect1->dataF[i] + s2*vect2->dataF[i];}}
 else if(vect->type == FR_VECT_C16)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataD[i] = s1*vect1->dataD[i] + s2*vect2->dataD[i];}}
 else if(vect->type == FR_VECT_2U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUS[i] = s1*vect1->dataUS[i] + s2*vect2->dataUS[i];}}
 else if(vect->type == FR_VECT_4U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUI[i] = s1*vect1->dataUI[i] + s2*vect2->dataUI[i];}}
 else if(vect->type == FR_VECT_8U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUL[i] = s1*vect1->dataUL[i] + s2*vect2->dataUL[i];}}
 else if(vect->type == FR_VECT_1U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataU[i] = s1*vect1->dataU[i] + s2*vect2->dataU[i];}}

 return(vect);
}

/*---------------------------------------------------------------FrvCombine--*/
FrVect *FrvCombine(int nVect,
                      ...)
/*---------------------------------------------------------------------------*/
/* Syntax: FrVect* = FrvCombine(int nVector, scale1, vect1, ...              */
/*                      ,scalen, vectn, FrVect* vectOut, char* newName);     */
/* This function computes vectOut = scale1*vect1+ ... up to n vectors        */
/* The vector vectOut could be any of the input vector. If vectOut = NULL,   */
/* a new one is created. Its name is newName.                                */
/* If vectOut != NULL, the newName argument is ignored.                      */
/*---------------------------------------------------------------------------*/
{va_list ap;
 FrVect *vect, **vectIn;
 double *scale;
 long i;
 char *name;

 if(nVect == 0) return(NULL);

 vectIn = (FrVect **) malloc(nVect * sizeof(FrVect*));
 scale  = (double *)  malloc(nVect * sizeof(double));
 if(scale == NULL || vectIn == NULL) return(NULL);

                    /*---------------- extract parameters -------------------*/

 va_start(ap, nVect);
 for(i=0; i<nVect; i++)
    {scale[i]  = va_arg(ap, double);
     vectIn[i] = va_arg(ap, FrVect*);}
 vect = va_arg(ap, FrVect*);
 name = va_arg(ap, char*);

                   /*--------------- perform operations --------------------*/
  
 if(nVect == 1)
   {vect = FrvScale(scale[i], vectIn[i], vect, name);}
else
   {vect = FrvCombine2(scale[0], vectIn[0], scale[0], vectIn[0], vect, name);
    if(nVect > 2)
    for(i=2; i<nVect; i++)
       {vect = FrvCombine2(1., vect, scale[i], vectIn[i], vect, NULL);}}

                   /*----------------- free local space----------------------*/

 free(vectIn);
 free(scale);

 return(vect);}

/*-----------------------------------------------------------------FrvDelta--*/
int FrvDelta(FrVect *vect, double *delta, double *previous)
/*---------------------------------------------------------------------------*/
/* This function computes maximum value of the differentiate vector:         */
/*       delta = max(abs(data[i+1] - data[i])).                              */
/* The result is return in delta. If previous != NULL, the corresponding     */
/* value is used to differentiate the first vector element. On return,       */
/* previous holds the value of the last vector element.                      */
/* This function returns a non zero value in case of error.                  */
/*---------------------------------------------------------------------------*/
{int i;
 double prev, val, diff;

 if(vect == NULL)     return(1);
 if(vect->nData == 0) return(1);

 *delta= 0.;
 val = 0;

 if(vect->type == FR_VECT_C)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->data[0];}
     for(i=0; i<vect->nData; i++) {val = vect->data[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_2S)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataS[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataS[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_8R)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataD[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataD[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_4R)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataF[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataF[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_4S)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataI[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataI[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_8S)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataL[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataL[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_C8)
    {return(0);}
 else if(vect->type == FR_VECT_C16)
    {return(0);}
 else if(vect->type == FR_VECT_2U)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataUS[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataUS[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_4U)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataUI[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataUI[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_8U)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataUL[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataUL[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else if(vect->type == FR_VECT_1U)
    {if(previous != NULL) 
          {prev = *previous;}
     else {prev = vect->dataU[0];}
     for(i=0; i<vect->nData; i++) {val = vect->dataU[i];
                                   diff = val - prev;
                                   if(diff < 0) diff = -diff;
				   if(diff > *delta) *delta = diff;
                                   prev = val;}}
 else{return(2);}
    
 if(previous != NULL) *previous = val;

 return(0);}

/*----------------------------------------------------------------FrvDivide--*/
FrVect *FrvDivide(FrVect *vect1,
                  FrVect *vect2,
                  FrVect *vect3,
                  char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes vectOut= vect1/vect2 elements by elements after    */
/* checking the division by 0. ( if the denominator is zero then the result  */
/* is set also to 0. The vector vectOut could be vect1 or vect2.             */
/* If vectOut = NULL, a new one is created called newName.                   */
/* This function returns, the vector result or NULL in case of error.        */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;
 float tempF;
 double tempD, denom;

 if(vect1 == NULL) return(NULL);
 if(vect2 == NULL) return(NULL);
 if(vect1->type  != vect2->type)  return(NULL);
 if(vect1->nData != vect2->nData) return(NULL);

 if(vect3 != NULL)
      {if(vect3->type  != vect1->type)  return(NULL);
       if(vect3->nData != vect1->nData) return(NULL);
       vect = vect3;}
 else {vect = FrvClone(vect1, newName);}

 if(vect->type == FR_VECT_C)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->data[i] == 0.) 
              vect->data[i] = 0.;
         else vect->data[i] = vect1->data[i] / vect2->data[i];}}
 else if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataS[i] == 0.) 
              vect->dataS[i] = 0.;
         else vect->dataS[i] = vect1->dataS[i] / vect2->dataS[i];}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataD[i] == 0.) 
              vect->dataD[i] = 0.;
         else vect->dataD[i] = vect1->dataD[i] / vect2->dataD[i];}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataF[i] == 0.) 
              vect->dataF[i] = 0.;
         else vect->dataF[i] = vect1->dataF[i] / vect2->dataF[i];}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataI[i] == 0.) 
              vect->dataI[i] = 0.;
         else vect->dataI[i] = vect1->dataI[i] / vect2->dataI[i];}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataL[i] == 0.) 
              vect->dataL[i] = 0.;
         else vect->dataL[i] = vect1->dataL[i] / vect2->dataL[i];}}
 else if(vect->type == FR_VECT_C8)
    {for(i=0; i<2*vect->nData; i=i+2)
        {denom = vect2->dataF[i]   * vect2->dataF[i] +
	         vect2->dataF[i+1] * vect2->dataF[i+1];
	 if(denom == 0)
	    {vect->dataF[i  ] = 0.;
             vect->dataF[i+1] = 0.;}
         else
            {tempF = (vect1->dataF[i]   * vect2->dataF[i] +
                      vect1->dataF[i+1] * vect2->dataF[i+1])/denom;
             vect->dataF[i+1] = (- vect1->dataF[i]   * vect2->dataF[i+1] +
                                   vect1->dataF[i+1] * vect2->dataF[i])/denom;
             vect->dataF[i] = tempF;}}}
 else if(vect->type == FR_VECT_C16)
    {for(i=0; i<2*vect->nData; i=i+2)
        {denom = vect2->dataD[i]   * vect2->dataD[i] +
	         vect2->dataD[i+1] * vect2->dataD[i+1];
	 if(denom == 0)
	    {vect->dataD[i  ] = 0.;
             vect->dataD[i+1] = 0.;}
         else
            {tempD = (vect1->dataD[i]   * vect2->dataD[i] +
                      vect1->dataD[i+1] * vect2->dataD[i+1])/denom;
             vect->dataD[i+1] = (- vect1->dataD[i]   * vect2->dataD[i+1] +
                                   vect1->dataD[i+1] * vect2->dataD[i])/denom;
             vect->dataD[i] = tempD;}}}
 else if(vect->type == FR_VECT_2U)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataUS[i] == 0.) 
              vect->dataUS[i] = 0.;
         else vect->dataUS[i] = vect1->dataUS[i] / vect2->dataUS[i];}}
 else if(vect->type == FR_VECT_4U)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataUI[i] == 0.) 
              vect->dataUI[i] = 0.;
         else vect->dataUI[i] = vect1->dataUI[i] / vect2->dataUI[i];}}
 else if(vect->type == FR_VECT_8U)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataUL[i] == 0.) 
              vect->dataUL[i] = 0.;
         else vect->dataUL[i] = vect1->dataUL[i] / vect2->dataUL[i];}}
 else if(vect->type == FR_VECT_1U)
    {for(i=0; i<vect->nData; i++)
        {if(vect2->dataU[i] == 0.) 
              vect->dataU[i] = 0.;
         else vect->dataU[i] = vect1->dataU[i] / vect2->dataU[i];}}

 return(vect);
}
/*---------------------------------------------------------------FrvFlatten--*/
FrVect *FrvFlatten(FrVect *vect1, FrVect *vectOut)
/*---------------------------------------------------------------------------*/
/*This function puts vector extrema equal to zero, subtracting a straigth    */
/*line.                                                                      */
/*If vectOut==NULL, vect1 is modified. Otherwise, a new vector with name     */
/*newName is created.                                                        */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;
 double a, b;
 if(vect1 == NULL) return(NULL);


 if(vectOut != NULL)
      {if(vectOut->type  != vect1->type)  return(NULL);
       if(vectOut->nData != vect1->nData) return(NULL);
       vect = vectOut;}
 else vect=vect1;
 /* output data type not supported! */
 if ((vect->type == FR_VECT_C) ||
     (vect->type == FR_VECT_8U)||
     (vect->type == FR_VECT_4U)||
     (vect->type == FR_VECT_2U)||
     (vect->type == FR_VECT_1U)) return (NULL);


 else if(vect->type == FR_VECT_2S)
   {b=vect1->dataS[0];
   a=(vect1->dataS[vect1->nData-1]-vect1->dataS[0])/(vect1->nData-1);
      for(i=0; i<vect->nData; i++)
        {vect->dataS[i] = vect1->dataS[i] - b-a*i;}}

 else if(vect->type == FR_VECT_8R)
   {b=vect1->dataD[0];
   a=(vect1->dataD[vect1->nData-1]-vect1->dataD[0])/(vect1->nData-1);
      for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] - b-a*i;}}

 else if(vect->type == FR_VECT_4R)
   {b=vect1->dataF[0];
   a=(vect1->dataF[vect1->nData-1]-vect1->dataF[0])/(vect1->nData-1);
      for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] - b-a*i;}}

 else if(vect->type == FR_VECT_4S)
   {b=vect1->dataI[0];
   a=(vect1->dataI[vect1->nData-1]-vect1->dataI[0])/(vect1->nData-1);
      for(i=0; i<vect->nData; i++)
        {vect->dataI[i] = vect1->dataI[i] - b-a*i;}}

 else if(vect->type == FR_VECT_8S)
   {b=vect1->dataL[0];
   a=(vect1->dataL[vect1->nData-1]-vect1->dataL[0])/(vect1->nData-1);
      for(i=0; i<vect->nData; i++)
        {vect->dataL[i] = vect1->dataL[i] - b-a*i;}}

 else if(vect->type == FR_VECT_C8)
   {b=vect1->dataF[0];
   a=(vect1->dataF[vect1->nData-1]-vect1->dataF[0])/(vect1->nData-1);
      for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] - b-a*i;}}

 else if(vect->type == FR_VECT_C16)
   {b=vect1->dataD[0];
   a=(vect1->dataD[vect1->nData-1]-vect1->dataD[0])/(vect1->nData-1);
      for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] - b-a*i;}}

 /* aggiungere il controllo sugli unsigned! */
 return(vect);
}

/*------------------------------------------------------------------FrvMean--*/
int FrvMean(FrVect *vect, double *mean)
/*---------------------------------------------------------------------------*/
/* This function computes the mean value of the input vector vect. The result*/
/* is put in mean. It returns 1 in case of failure or 0 in case of success.  */
/*---------------------------------------------------------------------------*/
{int i;
 double sum;

 if(vect == NULL)     return(1);
 if(vect->nData == 0) return(1);

 sum = 0.;
 if(vect->type == FR_VECT_C)
    {for(i=0; i<vect->nData; i++) {sum += vect->data[i];}}
 else if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataS[i];}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataD[i];}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataF[i];}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataI[i];}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataL[i];}}
 else if(vect->type == FR_VECT_C8)
    {return(0);}
 else if(vect->type == FR_VECT_C16)
    {return(0);}
 else if(vect->type == FR_VECT_2U)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataUS[i];}}
 else if(vect->type == FR_VECT_4U)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataUI[i];}}
 else if(vect->type == FR_VECT_8U)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataUL[i];}}
 else if(vect->type == FR_VECT_1U)
    {for(i=0; i<vect->nData; i++) {sum += vect->dataU[i];}}

 *mean = sum/vect->nData;

 return(0);}


/*---------------------------------------------------------------FrvModulus--*/
FrVect *FrvModulus(FrVect *vectIn,
                   FrVect *vectOut,
                   char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes the modulus of a complex vector vectIn.            */
/* The result is stored in vectOut, a real vector (Float or Double according */
/* to the complex type of vectIn). If vectOut = NULL, a new vector is created*/
/* Its name is newName or 'module(vectIn->name)' if newName = NULL.          */
/* It returns, the vector result or NULL in case of error.                   */
/*---------------------------------------------------------------------------*/
{char name[256];
 int i,j;

 if(vectIn == NULL) return(NULL);

 if(newName == NULL) 
   {sprintf(name,"module(%s)",vectIn->name);
    newName = name;}

 if(vectOut == NULL)
   {if(vectIn->type == FR_VECT_C8)
        {vectOut = FrVectNew1D(newName, FR_VECT_4R, vectIn->nData, 
                   vectIn->dx[0], vectIn->unitX[0], vectIn->unitY);}
    else if(vectIn->type == FR_VECT_C16)
        {vectOut = FrVectNew1D(newName, FR_VECT_8R, vectIn->nData, 
                   vectIn->dx[0], vectIn->unitX[0], vectIn->unitY);}}

 if(vectOut == NULL) return(NULL);
 if(vectOut->nData != vectIn->nData) return(NULL);

 if(vectIn->type == FR_VECT_C8)
    {if(vectOut->type != FR_VECT_4R) return(NULL);
     for(i=0; i<vectIn->nData; i++)
        {j = i+i;
         vectOut->dataF[i] = sqrt(vectIn->dataF[j]   * vectIn->dataF[j] +
                                  vectIn->dataF[j+1] * vectIn->dataF[j+1]);}}
 else 
    {if(vectOut->type != FR_VECT_8R) return(NULL);
     for(i=0; i<vectIn->nData; i++)
        {j = i+i;
         vectOut->dataD[i] = sqrt(vectIn->dataD[j]   * vectIn->dataD[j] +
                                  vectIn->dataD[j+1] * vectIn->dataD[j+1]);}}

 return(vectOut);
}
/*------------------------------------------------------------------FrvMult--*/
FrVect *FrvMult(FrVect *vect1,
                FrVect *vect2,
                FrVect *vect3,
                char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes vect3= vect1*vect2 elements by elements.           */
/* The vector vect3 could be vect1 or vect2. If vect3 = NULL a new vector    */
/* is created. Its name is then newName.                                     */
/* This function returns, the vector result or NULL in case of error.        */
/* If vectOut = NULL, the newName argument is ignored.                       */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;
 float tempF;
 double tempD;

 if(vect1 == NULL) return(NULL);
 if(vect2 == NULL) return(NULL);
 if(vect1->type  != vect2->type)  return(NULL);
 if(vect1->nData != vect2->nData) return(NULL);

 if(vect3 != NULL)
      {if(vect3->type  != vect1->type)  return(NULL);
       if(vect3->nData != vect1->nData) return(NULL);
       vect = vect3;}
 else {vect = FrvClone(vect1, newName);}

 if(vect->type == FR_VECT_C)
    {for(i=0; i<vect->nData; i++)
        {vect->data[i] = vect1->data[i] * vect2->data[i];}}
 else if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataS[i] = vect1->dataS[i] * vect2->dataS[i];}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] * vect2->dataD[i];}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] * vect2->dataF[i];}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataI[i] = vect1->dataI[i] * vect2->dataI[i];}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataL[i] = vect1->dataL[i] * vect2->dataL[i];}}
 else if(vect->type == FR_VECT_C8)
    {for(i=0; i<2*vect->nData; i=i+2)
        {tempF = vect1->dataF[i]   * vect2->dataF[i] -
                 vect1->dataF[i+1] * vect2->dataF[i+1];
         vect->dataF[i+1] = vect1->dataF[i]   * vect2->dataF[i+1] +
                            vect1->dataF[i+1] * vect2->dataF[i];
         vect->dataF[i] = tempF;}}
 else if(vect->type == FR_VECT_C16)
    {for(i=0; i<2*vect->nData; i=i+2)
        {tempD = vect1->dataD[i]   * vect2->dataD[i] -
                 vect1->dataD[i+1] * vect2->dataD[i+1];
         vect->dataD[i+1] = vect1->dataD[i]   * vect2->dataD[i+1] +
                            vect1->dataD[i+1] * vect2->dataD[i];
         vect->dataD[i] = tempD;}}
 else if(vect->type == FR_VECT_2U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUS[i] = vect1->dataUS[i] * vect2->dataUS[i];}}
 else if(vect->type == FR_VECT_4U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUI[i] = vect1->dataUI[i] * vect2->dataUI[i];}}
 else if(vect->type == FR_VECT_8U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUL[i] = vect1->dataUL[i] * vect2->dataUL[i];}}
 else if(vect->type == FR_VECT_1U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataU[i] = vect1->dataU[i] * vect2->dataU[i];}}

 return(vect);
}
/*--------------------------------------------------------------FrvMultConj--*/
FrVect *FrvMultConj(FrVect *vect1,
                    FrVect *vect2,
                    FrVect *vect3,
                    char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes vect3= vect1*cc(vect2) elements by elements.       */
/* The vector vect3 could be vect1 or vect2. If vect3 = NULL a new vector    */
/* is created. Its name is then newName.                                     */
/* This function returns, the vector result or NULL in case of error.        */
/* If vectOut = NULL, the newName argument is ignored.                       */
/* Supported types are FR_VECT_C8 and FR_VECT_C16.                           */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;
 float tempF;
 double tempD;

 if(vect1 == NULL) return(NULL);
 if(vect2 == NULL) return(NULL);
 if(vect1->type  != vect2->type)  return(NULL);
 if(vect1->nData != vect2->nData) return(NULL);

 if(vect3 != NULL)
      {if(vect3->type  != vect1->type)  return(NULL);
       if(vect3->nData != vect1->nData) return(NULL);
       vect = vect3;}
 else {vect = FrvClone(vect1, newName);}

 if(vect->type == FR_VECT_C8)
    {for(i=0; i<2*vect->nData; i=i+2)
        {tempF = vect1->dataF[i]   * vect2->dataF[i] +
                 vect1->dataF[i+1] * vect2->dataF[i+1];
         vect->dataF[i+1] = - vect1->dataF[i]   * vect2->dataF[i+1] +
                              vect1->dataF[i+1] * vect2->dataF[i];
         vect->dataF[i] = tempF;}}
 else if(vect->type == FR_VECT_C16)
    {for(i=0; i<2*vect->nData; i=i+2)
        {tempD = vect1->dataD[i]   * vect2->dataD[i] +
                 vect1->dataD[i+1] * vect2->dataD[i+1];
         vect->dataD[i+1] = - vect1->dataD[i]   * vect2->dataD[i+1] +
                              vect1->dataD[i+1] * vect2->dataD[i];
         vect->dataD[i] = tempD;}}
 else 
   {if(vect3 == NULL) free(vect);
    return(NULL);}

 return(vect);
}

/*-----------------------------------------------------------------FrvPhase--*/
FrVect *FrvPhase(FrVect *vectIn,
                 FrVect *vectOut,
                 char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes the phase of a complex vector vectIn.              */
/* The result is stored in vectOut, a real vector (Float or Double according */
/* to the complex type of vectIn). If vectOut = NULL, a new one is created.  */
/* Its name is newName or "phase(vectIn->name)" if newName = NULL.           */
/* It returns, the vector result or NULL in case of error.                   */
/*---------------------------------------------------------------------------*/
{char name[256];
 int i;

 if(vectIn == NULL) return(NULL);

 if(newName == NULL) 
   {sprintf(name,"phase(%s)",vectIn->name);
    newName = name;}

 if(vectOut == NULL)
   {if(vectIn->type == FR_VECT_C8)
        {vectOut = FrVectNew1D(newName, FR_VECT_4R, vectIn->nData, 
                   vectIn->dx[0], vectIn->unitX[0], vectIn->unitY);}
    else if(vectIn->type == FR_VECT_C16)
        {vectOut = FrVectNew1D(newName, FR_VECT_8R, vectIn->nData, 
                   vectIn->dx[0], vectIn->unitX[0], vectIn->unitY);}}

 if(vectOut == NULL) return(NULL);
 if(vectOut->nData != vectIn->nData) return(NULL);

 if(vectIn->type == FR_VECT_C8)
    {if(vectOut->type != FR_VECT_4R) return(NULL);
     for(i=0; i<vectIn->nData; i++)
        {vectOut->dataF[i] = atan2(vectIn->dataF[i+i+1],vectIn->dataF[i+i]);}}
 else 
    {if(vectOut->type != FR_VECT_8R) return(NULL);
     for(i=0; i<vectIn->nData; i++)
        {vectOut->dataD[i] = atan2(vectIn->dataD[i+i+1],vectIn->dataD[i+i]);}}

 return(vectOut);
}
/*-------------------------------------------------------------------FrvRms--*/
int FrvRms(FrVect *vect, double *mean, double *rms)
/*---------------------------------------------------------------------------*/
/* This function computes the rms and mean value of the input vector vect.   */
/* The result is put in mean and rms. It returns 0 in case of success.       */
/*---------------------------------------------------------------------------*/
{int i;
 double sum, sum2, val, rms2;

 if(vect == NULL)     return(1);
 if(vect->nData == 0) return(2);

 sum  = 0.;
 sum2 = 0.;

 if(vect->type == FR_VECT_C)
    {for(i=0; i<vect->nData; i++) {val = vect->data[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++) {val = vect->dataS[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++) {val = vect->dataD[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++) {val = vect->dataF[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++) {val = vect->dataI[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++) {val = vect->dataL[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_C8)
    {return(3);}
 else if(vect->type == FR_VECT_C16)
    {return(3);}
 else if(vect->type == FR_VECT_2U)
    {for(i=0; i<vect->nData; i++) {val = vect->dataUS[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_4U)
    {for(i=0; i<vect->nData; i++) {val = vect->dataUI[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_8U)
    {for(i=0; i<vect->nData; i++) {val = vect->dataUL[i];
                                   sum  += val;
                                   sum2 += val*val;}}
 else if(vect->type == FR_VECT_1U)
    {for(i=0; i<vect->nData; i++) {val = vect->dataU[i];
                                   sum  += val;
                                   sum2 += val*val;}}

 *mean = sum/vect->nData;
 rms2 = sum2/vect->nData - (*mean)*(*mean);
 *rms = sqrt(rms2);

 return(0);}

/*-----------------------------------------------------------------FrvScale--*/
FrVect *FrvScale(double scale,
                 FrVect *vect1,
                 FrVect *vect3,
                 char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes vect3 = scale*vect1 elements by elements.          */
/* The vector vect3 could be vect1 or vect2.                                 */
/* If vect3 = NULL, a new one is created. Its name is then newName.          */
/* It returns, the vector result or NULL in case of error.                   */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i, scaleI;
 char   scaleC;
 short  scaleS;
 long   scaleL;
 float  scaleF;
 double scaleD;
 unsigned char  scaleU;
 unsigned short scaleUS;
 unsigned int   scaleUI;
 unsigned long  scaleUL;

 if(vect1 == NULL) return(NULL);

 if(vect3 != NULL)
      {if(vect3->type  != vect1->type)  return(NULL);
       if(vect3->nData != vect1->nData) return(NULL);
       vect = vect3;}
 else {vect = FrvClone(vect1,newName);}

 if(vect->type == FR_VECT_C)
    {scaleC = scale;
     for(i=0; i<vect->nData; i++) 
        {vect->data[i] = scale * vect1->data[i];}}
 else if(vect->type == FR_VECT_2S)
    {scaleS = scale;
     for(i=0; i<vect->nData; i++)
        {vect->dataS[i] = scaleS * vect1->dataS[i];}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = scale * vect1->dataD[i];}}
 else if(vect->type == FR_VECT_4R)
    {scaleF = scale;
     for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = scaleF * vect1->dataF[i];}}
 else if(vect->type == FR_VECT_4S)
    {scaleI = scale;
     for(i=0; i<vect->nData; i++)
        {vect->dataI[i] = scaleI * vect1->dataI[i];}}
 else if(vect->type == FR_VECT_8S)
    {scaleL = scale;
     for(i=0; i<vect->nData; i++)
        {vect->dataL[i] = scaleL * vect1->dataL[i];}}
 else if(vect->type == FR_VECT_C8 || vect->type == FR_VECT_H8)
    {scaleF = scale;
     for(i=0; i<2*vect->nData; i++)
        {vect->dataF[i] = scaleF * vect1->dataF[i];}}
 else if(vect->type == FR_VECT_C16 || vect->type == FR_VECT_H16)
    {scaleD = scale;
     for(i=0; i<2*vect->nData; i++)
        {vect->dataD[i] = scaleD * vect1->dataD[i];}}
 else if(vect->type == FR_VECT_2U)
    {scaleUS = scale;
     for(i=0; i<vect->nData; i++)
        {vect->dataUS[i] = scaleUS * vect1->dataUS[i];}}
 else if(vect->type == FR_VECT_4U)
    {scaleUI = scale;
     for(i=0; i<vect->nData; i++)
        {vect->dataUI[i] = scaleUI * vect1->dataUI[i];}}
 else if(vect->type == FR_VECT_8U)
    {scaleUL = scale;
     for(i=0; i<vect->nData; i++)
        {vect->dataUL[i] = scaleUL * vect1->dataUL[i];}}
 else if(vect->type == FR_VECT_1U)
    {scaleU = scale;
     for(i=0; i<vect->nData; i++)
        {vect->dataU[i] = scaleU * vect1->dataU[i];}}

 return(vect);
}

/*-------------------------------------------------------------------FrvSub--*/
FrVect *FrvSub(FrVect *vect1,
               FrVect *vect2,
               FrVect *vect3,
                      char *newName)
/*---------------------------------------------------------------------------*/
/* This function computes vect3= vect1- vect2.                               */
/* The vector vect3 could be vect1 or vect2.                                 */
/* If vect3 = NULL, a new one is created. Its name is then newName.          */
/* It returns, the vector result or NULL in case of error.                   */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;

 if(vect1 == NULL) return(NULL);
 if(vect2 == NULL) return(NULL);
 if(vect1->type  != vect2->type)  return(NULL);
 if(vect1->nData != vect2->nData) return(NULL);

 if(vect3 != NULL)
      {if(vect3->type  != vect1->type)  return(NULL);
       if(vect3->nData != vect1->nData) return(NULL);
       vect = vect3;}
 else {vect = FrvClone(vect1, newName);}

 if(vect->type == FR_VECT_C)
    {for(i=0; i<vect->nData; i++)
        {vect->data[i] = vect1->data[i] - vect2->data[i];}}
 else if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataS[i] = vect1->dataS[i] - vect2->dataS[i];}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] - vect2->dataD[i];}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] - vect2->dataF[i];}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataI[i] = vect1->dataI[i] - vect2->dataI[i];}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataL[i] = vect1->dataL[i] - vect2->dataL[i];}}
 else if(vect->type == FR_VECT_C8)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] - vect2->dataF[i];}}
 else if(vect->type == FR_VECT_C16)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] - vect2->dataD[i];}}
 else if(vect->type == FR_VECT_2U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUS[i] = vect1->dataUS[i] - vect2->dataUS[i];}}
 else if(vect->type == FR_VECT_4U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUI[i] = vect1->dataUI[i] - vect2->dataUI[i];}}
 else if(vect->type == FR_VECT_8U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataUL[i] = vect1->dataUL[i] - vect2->dataUL[i];}}
 else if(vect->type == FR_VECT_1U)
    {for(i=0; i<vect->nData; i++)
        {vect->dataU[i] = vect1->dataU[i] - vect2->dataU[i];}}

 return(vect);
}

/*--------------------------------------------------------------FrvZeroMean--*/
FrVect *FrvZeroMean(FrVect *vect1, FrVect *vectOut) 
/*---------------------------------------------------------------------------*/
/* This function computes vectOut= vect1+vect2. The vector vectOut could be  */
/* vect1 or vect2. If vectOut = NULL, a new on.    */
/* It returns, the output vector or NULL in case of error.                   */
/* If vectOut = NULL, the newName argument is ignored.                       */
/*---------------------------------------------------------------------------*/
{FrVect *vect;
 int i;
 double mean;
 if(vect1 == NULL) return(NULL);


 if(vectOut != NULL)
      {if(vectOut->type  != vect1->type)  return(NULL);
       if(vectOut->nData != vect1->nData) return(NULL);
       vect = vectOut;}
 else vect=vect1;
 if ((vect->type == FR_VECT_C) ||
     (vect->type == FR_VECT_8U)||
     (vect->type == FR_VECT_4U)||
     (vect->type == FR_VECT_2U)||
     (vect->type == FR_VECT_1U)) return (NULL);

 FrvMean(vect1,&mean);

 if(vect->type == FR_VECT_2S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataS[i] = vect1->dataS[i] - mean;}}
 else if(vect->type == FR_VECT_8R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i]- mean;}}
 else if(vect->type == FR_VECT_4R)
    {for(i=0; i<vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] - mean;}}
 else if(vect->type == FR_VECT_4S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataI[i] = vect1->dataI[i]- mean;}}
 else if(vect->type == FR_VECT_8S)
    {for(i=0; i<vect->nData; i++)
        {vect->dataL[i] = vect1->dataL[i] - mean;}}
 else if(vect->type == FR_VECT_C8)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataF[i] = vect1->dataF[i] - mean;}}
 else if(vect->type == FR_VECT_C16)
    {for(i=0; i<2*vect->nData; i++)
        {vect->dataD[i] = vect1->dataD[i] - mean;}}

 return(vect);
}
